program Sprite_compressor;
  {vypyta si meno archivu (.csp) kam pojde vystup, potom pyta subory s jednym
   spritom (.spr) alebo blokom spritov (.sps) (ak existuju obidva vezme .spr)
   a vyskusa ho skompresit RLE, LZW a RLE+LZW, vyberie najlepsiu
   moznost prida na koniec archivu a vypise jeho poziciu od zaciatku suboru,
   ratio a pouzitu kompresiu
  }

  {CSP (compressed sprites) format - subor sa sklada z blokov tvaru:
    size:longint;               - pocet bytov neskompreseneho bloku spritov
    csize:longint;              - pocet bytov skompreseneho bloku spritov
    status:word;                - info
    data:array[csize] of byte   - samotny skompreseny blok spritov

   status: x x x x x x x x  x x x x x x x x  0/1 ne/pouzita RLE kompresia
                                       0/1 ne/pouzita LZW kompresia
                                     
                                        . . .  nevyuzite
           
  }

  {$G+}

  uses Dos;

  type spritemap=array[0..65520] of byte;
       pspritemap=^spritemap;
       pbyte=^byte;

  var sprpath,archpath:pathstr;

      sprite,rlesprite,lzwsprite,rlelzwsprite,minsprite,buf1,buf2:pspritemap;
      p1,p2,p3,p4,p5,p6:pbyte;
      fin,fout:file of byte;
      dx,dy,i,size,rlesize,lzwsize,rlelzwsize,minsize,
      rlestatus,lzwstatus,rlelzwstatus,minstatus:word;
      b1,b2:byte;

  {$I compress.inc}


  function AdjustFilename(var path:pathstr; ext2,ext3:extstr):boolean;
    var dir:dirstr;
        name:namestr;
        ext:extstr;
    begin
      FSplit(path,dir,name,ext);
      if ext='' then ext:=ext2;
      if (ext<>ext2) and (ext<>ext3) then
        begin
          Write('Incorrect extension, must be ',ext2);
          if ext2<>ext3 then Write(' or ',ext3);
          WriteLn; WriteLn;
          AdjustFilename:=false;
        end
      else
        begin path:=dir+name+ext; AdjustFilename:=true; end;
    end;


  procedure GetMem0(var pvar:pspritemap; size:word; var key:pbyte);
    begin
      key:=nil; GetMem(pvar,size);
      while Ofs(pvar^)<>0 do
        begin FreeMem(pvar,size); New(key); GetMem(pvar,size); end;
    end;


  procedure FreeMem0(pvar:pspritemap; size:word; key:pbyte);
    begin
      if key<>nil then Dispose(key);
      FreeMem(pvar,size);
    end;


  procedure ComputeGlobalRatio;
    var nbofblocks,globalsize,globalcsize:longint;
    begin
      Assign(fin,archpath); Reset(fin);
      nbofblocks:=0; globalsize:=0; globalcsize:=0;

      while not EoF(fin) do
        begin
          Read(fin,b1,b2); size:=b2*256+b1;
          Read(fin,b1,b2);
          Read(fin,b1,b2); rlelzwsize:=b2*256+b1;
          Read(fin,b1,b1,b1,b1);
          for i:=0 to rlelzwsize-1 do Read(fin,b1);
          Inc(nbofblocks);
          Inc(globalsize,size);
          Inc(globalcsize,rlelzwsize);
        end;
      WriteLn('blocks of sprites in archive: ',nbofblocks);
      WriteLn('original archive size: ',globalsize,
              ',  compressed archive size: ',globalcsize);
      Write('archive ratio: ');
      if globalsize>0 then WriteLn(Trunc(globalcsize/globalsize*100),'%')
      else WriteLn;
      WriteLn;
    end;


  procedure Compress(sprpath:pathstr);
    var sprpath2:pathstr;
    begin
      if not AdjustFilename(sprpath,'.spr','.sps') then Exit;
      Assign(fin,sprpath); {$I-} Reset(fin); {$I+}
      if IOResult<>0 then
        begin
          WriteLn(sprpath,' not found'); WriteLn; Exit;
        end;

      size:=FileSize(fin);

      GetMem0(sprite,size,p1);
      for i:=0 to size-1 do Read(fin,sprite^[i]);
      Close(fin);

      GetMem0(rlesprite,size,p2);
      rlestatus:=0;
      RLECompress(rlesprite,sprite,rlesize,size,rlestatus);

      GetMem0(lzwsprite,size,p3);
      GetMem0(buf1,65520,p5); GetMem0(buf2,65520,p6);
      lzwstatus:=0;
      LZWCompress(lzwsprite,sprite,buf1,buf2,lzwsize,size,lzwstatus);

      GetMem0(rlelzwsprite,size,p4);
      rlelzwstatus:=rlestatus;
      LZWCompress(rlelzwsprite,rlesprite,buf1,buf2,rlelzwsize,rlesize,rlelzwstatus);
      FreeMem0(buf1,65520,p5); FreeMem0(buf2,65520,p6);


      minsprite:=sprite; minsize:=size; minstatus:=0;
      if (rlestatus and 1)=1 then
        begin minsprite:=rlesprite; minsize:=rlesize; minstatus:=rlestatus; end;
      if ((lzwstatus and 2)=2) and (lzwsize<minsize) then
        begin minsprite:=lzwsprite; minsize:=lzwsize; minstatus:=lzwstatus; end;
      if ((rlelzwstatus and 3)=3) and (rlelzwsize<minsize) then
        begin
          minsprite:=rlelzwsprite; minsize:=rlelzwsize; minstatus:=rlelzwstatus;
        end;

      Assign(fout,archpath); Reset(fout);
      Seek(fout,FileSize(fout));
      WriteLn('block position: ',FilePos(fout));
      WriteLn('original size: ',size,',  ',
              'compressed size: ',minsize,',  ',
              'ratio: ',Trunc(minsize/size*100),'%');
      Write('used compression: ');
      if (minstatus and 1)=1 then Write('RLE ');
      if (minstatus and 2)=2 then Write('LZW');
      WriteLn; WriteLn;

      b1:=size mod 256; b2:=size div 256 mod 256; Write(fout,b1,b2);
      b1:=size div 65536 mod 256; b2:=dy div 16777216 mod 256; Write(fout,b1,b2);
      b1:=minsize mod 256; b2:=minsize div 256 mod 256; Write(fout,b1,b2);
      b1:=minsize div 65536 mod 256; b2:=minsize div 16777216; Write(fout,b1,b2);
      b1:=minstatus mod 256; b2:=minstatus div 256; Write(fout,b1,b2);
      for i:=0 to minsize-1 do Write(fout,minsprite^[i]);
      Close(fout);

      FreeMem0(rlelzwsprite,size,p4); FreeMem0(lzwsprite,size,p3);
      FreeMem0(rlesprite,size,p2); FreeMem0(sprite,size,p1);
    end;


  begin
    WriteLn('Enter [path\]archname[.csp] of archive file');
    ReadLn(archpath);
    if not AdjustFilename(archpath,'.csp','.csp') then Exit;
    Assign(fin,archpath); {$I-} Reset(fin); {$I+}
    if IOResult<>0 then
        begin
          {$I-} Rewrite(fin); {$I+}
          if IOResult<>0 then
            begin
              WriteLn('Invalid path ',archpath,
                      ', cannot create new archive file');
              WriteLn;
              Exit;
            end
          else WriteLn(archpath,' not found, created new');
        end;
    Close(fin);
    WriteLn;

    repeat
      WriteLn('Enter [path\]sprname[.spr|.sps] of compressed block of '+
              'sprites, or keyword ''RATIO'' for global ratio of archive,'+
              ' or ''EXIT'' for exit compressor');
      ReadLn(sprpath);

      if sprpath='RATIO' then ComputeGlobalRatio
      else if sprpath<>'EXIT' then Compress(sprpath);

    until sprpath='EXIT';
end.
