{$N+} {$E+}
unit Vector;

  interface

    type tvector=record
                  x,y,z:single;
                end;

    type tmatrix=array[1..4,1..4] of single;

    const onematrix:tmatrix=((1,0,0,0),(0,1,0,0),(0,0,1,0),(0,0,0,1));
      {jednotkova matica}

    procedure CreateTranslationMatrix(var a:tmatrix; var v:tvector);
    procedure CreateXRotationMatrix(var a:tmatrix; angle:single);
    procedure CreateYRotationMatrix(var a:tmatrix; angle:single);
    procedure CreateZRotationMatrix(var a:tmatrix; angle:single);
    procedure MultiplyMatrices(var a,b:tmatrix);
    procedure MultiplyVectorMatrix(var u,v:tvector; var a:tmatrix);
    procedure VLoad(var v:tvector; x,y,z:single);
    procedure VAdd(var u,v,w:tvector);
    procedure VInc(var u,v:tvector);
    procedure VSub(var u,v,w:tvector);
    procedure VDec(var u,v:tvector);
    procedure VRMul(var u,v:tvector; r:single);
    function VLength(var u:tvector):single;
    procedure VTrunc(u,v:tvector);
    function VDotProduct(var u,v:tvector):single;
    function VCosAngle(var u,v:tvector):single;
    procedure VCrossProduct(var u,v,w:tvector);
    procedure TransformFromBase(var u,v,v1x,v1y,v1z:tvector);
    procedure VSplitToAngles(var v:tvector; var fi,sigma,p:single);
    procedure VComputeFromAngles(var v:tvector; var fi,sigma,p:single);


  implementation

    function ArcSin(x:single):single;
      begin
        ArcSin:=ArcTan(x/Sqrt(1-Sqr(x)));
      end;


    procedure CreateTranslationMatrix(var a:tmatrix; var v:tvector);
      {vytvori maticu posunutia o vektor v}
      begin
        a:=onematrix;
        a[4,1]:=v.x;
        a[4,2]:=v.y;
        a[4,3]:=v.z;
      end;


    procedure CreateXRotationMatrix(var a:tmatrix; angle:single);
      {vytvori maticu otocenia o uhol angle okolo osi x}
      begin
        a:=onematrix;
        a[2,2]:=Cos(angle);
        a[2,3]:=Sin(angle);
        a[3,2]:=-Sin(angle);
        a[3,3]:=Cos(angle);
      end;


    procedure CreateYRotationMatrix(var a:tmatrix; angle:single);
      {vytvori maticu otocenia o uhol angle okolo osi y}
      begin
        a:=onematrix;
        a[1,1]:=Cos(angle);
        a[1,3]:=Sin(angle);
        a[3,1]:=-Sin(angle);
        a[3,3]:=Cos(angle);
      end;


    procedure CreateZRotationMatrix(var a:tmatrix; angle:single);
      {vytvori maticu otocenia o uhol angle okolo osi z}
      begin
        a:=onematrix;
        a[1,1]:=Cos(angle);
        a[1,2]:=Sin(angle);
        a[2,1]:=-Sin(angle);
        a[2,2]:=Cos(angle);
      end;


    procedure MultiplyMatrices(var a,b:tmatrix);
      {vynasobi maticu a maticou b sprava a vysledok ulozi do matice a}
      var i,j:integer;
          tmp:array[1..4] of single;
      begin
        for i:=1 to 4 do
          begin
            for j:=1 to 4 do
              tmp[j]:=a[i,1]*b[1,j]+a[i,2]*b[2,j]+a[i,3]*b[3,j]+a[i,4]*b[4,j];
            for j:=1 to 4 do
              a[i,j]:=tmp[j];
          end;
      end;


    procedure MultiplyVectorMatrix(var u,v:tvector; var a:tmatrix);
      {vynasobi vektor v maticou a spravaa vysledok ulozi do vektora u}
      var vx,vy,vz:single;
      begin
        vx:=v.x; vy:=v.y; vz:=v.z;
        u.x:=vx*a[1,1]+vy*a[2,1]+vz*a[3,1]+a[4,1];
        u.y:=vx*a[1,2]+vy*a[2,2]+vz*a[3,2]+a[4,2];
        u.z:=vx*a[1,3]+vy*a[2,3]+vz*a[3,3]+a[4,3];
      end;


    procedure VLoad(var v:tvector; x,y,z:single);
      begin
        v.x:=x;
        v.y:=y;
        v.z:=z;
      end;


    procedure VAdd(var u,v,w:tvector);
      {scita vektory v a w a vysledok ulozi do vektora u}
      begin
        u.x:=v.x+w.x;
        u.y:=v.y+w.y;
        u.z:=v.z+w.z;
      end;


    procedure VInc(var u,v:tvector);
      {k vektoru u pripocita vektor v}
      begin
        VAdd(u,u,v);
      end;


    procedure VSub(var u,v,w:tvector);
      {odcita vektor w od vektora v a vysledok ulozi do vektora u}
      begin
        u.x:=v.x-w.x;
        u.y:=v.y-w.y;
        u.z:=v.z-w.z;
      end;


    procedure VDec(var u,v:tvector);
      {od vektora u odcita vektor v}
      begin
        VSub(u,u,v);
      end;


    procedure VRMul(var u,v:tvector; r:single);
      {vynasobi vektor v realnym cislom r a vysledok ulozi do u}
      begin
        u.x:=r*v.x;
        u.y:=r*v.y;
        u.z:=r*v.z;
      end;


    function VLength(var u:tvector):single;
      {vypocita dlzku vektora}
      begin
        VLength:=Sqrt(Sqr(u.x)+Sqr(u.y)+Sqr(u.z));
      end;


    procedure VTrunc(u,v:tvector);
      {do u vlozi celu cast vektora v}
      begin
        u.x:=Trunc(v.x);
        u.y:=Trunc(v.y);
        u.z:=Trunc(v.z);
      end;


    function VDotProduct(var u,v:tvector):single;
      {vypocita skalarny sucin vektorov u,v}
      begin
        VDotProduct:=u.x*v.x+u.y*v.y+u.z*v.z;
      end;


    function VCosAngle(var u,v:tvector):single;
      {vypocita kosinus uhla medzi vektormi u,v}
      begin
        VCosAngle:=VDotProduct(u,v)/(VLength(u)*VLength(v));
      end;


    procedure VCrossProduct(var u,v,w:tvector);
      {vektorovy sucin vektorov v,w ulozi do vektora u}
      var vx,vy,vz,wx,wy,wz:single;
      begin
        vx:=v.x; vy:=v.y; vz:=v.z;
        wx:=w.x; wy:=w.y; wz:=w.z;
        u.x:=vy*wz-vz*wy;
        u.y:=vz*wx-vx*wz;
        u.z:=vx*wy-vy*wx;
      end;


    procedure TransformFromBase(var u,v,v1x,v1y,v1z:tvector);
      {vo vektore u vrati suradnice vektora v, v baze (v1x,v1y,v1z)}
      begin
        u.x:=v.x*v1x.x+v.y*v1y.x+v.z*v1z.x;
        u.y:=v.x*v1x.y+v.y*v1y.y+v.z*v1z.y;
        u.z:=v.x*v1x.z+v.y*v1y.z+v.z*v1z.z;
      end;


    procedure VSplitToAngles(var v:tvector; var fi,sigma,p:single);
      {rozlozi vektor v na uhol fi (v xz) a sigma (s xz) a jeho dlzku p}
      begin
        p:=VLength(v);
        if p=0 then Exit;

        if v.x=0 then
          if v.z>0 then fi:=Pi/2
          else fi:=-Pi/2
        else fi:=ArcTan(v.z/v.x);
        if v.x<0 then fi:=fi+Pi;

        if v.y/p>=1 then sigma:=Pi/2
        else if v.y/p<=-1 then sigma:=-Pi/2
        else sigma:=ArcSin(v.y/p);
      end;


    procedure VComputeFromAngles(var v:tvector; var fi,sigma,p:single);
      {vypocita suradnice vektora v z uhlov fi, sigma a jeho dlzky p}
      begin
        v.x:=p*Cos(fi)*Cos(sigma);
        v.y:=p*Sin(sigma);
        v.z:=p*Sin(fi)*Cos(sigma);
      end;

end.

